@@ -593,7 +593,7 @@ end
 -- the current target is belong to the given platforms?
 function _instance:is_plat(...)
     local plat = self:plat()
-    for _, v in ipairs(table.join(...)) do
+    for _, v in ipairs(table.pack(...)) do
         if v and plat == v then
             return true
         end
@@ -603,7 +603,7 @@ end
 -- the current target is belong to the given architectures?
 function _instance:is_arch(...)
     local arch = self:arch()
-    for _, v in ipairs(table.join(...)) do
+    for _, v in ipairs(table.pack(...)) do
         if v and arch:find("^" .. v:gsub("%-", "%%-") .. "$") then
             return true
         end
@@ -2092,11 +2092,7 @@ end
 -- @param langkind  c/cxx
 --
 function _instance:pcoutputfile(langkind)
-
-    -- init cache
     self._PCOUTPUTFILES = self._PCOUTPUTFILES or {}
-
-    -- get it from the cache first
     local pcoutputfile = self._PCOUTPUTFILES[langkind]
     if pcoutputfile then
         return pcoutputfile
@@ -2108,7 +2104,9 @@ function _instance:pcoutputfile(langkind)
 
         -- is gcc?
         local is_gcc = false
-        local _, toolname = self:tool(langkind == "c" and "cc" or "cxx")
+        local sourcekinds = {c = "cc", cxx = "cxx", m = "mm", mxx = "mxx"}
+        local sourcekind = assert(sourcekinds[langkind], "unknown language kind: " .. langkind)
+        local _, toolname = self:tool(sourcekind)
         if toolname and (toolname == "gcc" or toolname == "gxx") then
             is_gcc = true
         end
@@ -2118,9 +2116,7 @@ function _instance:pcoutputfile(langkind)
         -- @note gcc has not -include-pch option to set the pch file path
         --
         pcoutputfile = self:objectfile(pcheaderfile)
-        pcoutputfile = path.join(path.directory(pcoutputfile), path.basename(pcoutputfile) .. (is_gcc and ".gch" or ".pch"))
-
-        -- save to cache
+        pcoutputfile = path.join(path.directory(pcoutputfile), sourcekind, path.basename(pcoutputfile) .. (is_gcc and ".gch" or ".pch"))
         self._PCOUTPUTFILES[langkind] = pcoutputfile
         return pcoutputfile
     end
@@ -2234,6 +2230,20 @@ function _instance:toolconfig(name)
     end})
 end
 
+-- has source files with the given source kind?
+function _instance:has_sourcekind(...)
+    local sourcekinds_set = self._SOURCEKINDS_SET
+    if sourcekinds_set == nil then
+        sourcekinds_set = hashset.from(self:sourcekinds())
+        self._SOURCEKINDS_SET = sourcekinds_set
+    end
+    for _, v in ipairs(table.pack(...)) do
+        if sourcekinds_set:has(v) then
+            return true
+        end
+    end
+end
+
 -- has the given tool for the current target?
 --
 -- e.g.
@@ -2244,7 +2254,7 @@ end
 function _instance:has_tool(toolkind, ...)
     local _, toolname = self:tool(toolkind)
     if toolname then
-        for _, v in ipairs(table.join(...)) do
+        for _, v in ipairs(table.pack(...)) do
             if v and toolname:find("^" .. v:gsub("%-", "%%-") .. "$") then
                 return true
             end
