@@ -32,7 +32,6 @@ import("utils.progress")
 import("private.cache.build_cache")
 import("private.service.distcc_build.client", {alias = "distcc_build_client"})
 
--- init it
 function init(self)
 
     -- init mxflags
@@ -44,16 +43,6 @@ function init(self)
     -- init shflags
     self:set("shflags", "-shared")
 
-    -- add -fPIC for shared
-    --
-    -- we need check it for clang/gcc with window target
-    -- @see https://github.com/xmake-io/xmake/issues/1392
-    --
-    if not self:is_plat("windows", "mingw") and self:has_flags("-fPIC", "cxflags") then
-        self:add("shflags", "-fPIC")
-        self:add("shared.cxflags", "-fPIC")
-    end
-
     -- init flags map
     self:set("mapflags", {
         -- warnings
@@ -76,6 +65,21 @@ function init(self)
     end
 end
 
+-- we can only call has_flags in load(),
+-- as it requires the full platform toolchain flags.
+--
+function load(self)
+    -- add -fPIC for shared
+    --
+    -- we need check it for clang/gcc with window target
+    -- @see https://github.com/xmake-io/xmake/issues/1392
+    --
+    if not self:is_plat("windows", "mingw") and self:has_flags("-fPIC") then
+        self:add("shflags", "-fPIC")
+        self:add("shared.cxflags", "-fPIC")
+    end
+end
+
 -- make the strip flag
 function nf_strip(self, level, target)
     local maps = {
@@ -84,6 +88,9 @@ function nf_strip(self, level, target)
     }
     if self:is_plat("macosx", "iphoneos") then
         maps.all = {"-Wl,-x", "-Wl,-dead_strip"}
+    elseif self:is_plat("windows") then
+        -- clang does not it on windows, TODO maybe we need test it for gcc
+        maps = {}
     end
     return maps[level]
 end
@@ -166,6 +173,7 @@ function nf_vectorext(self, extension)
     ,   ssse3 = "-mssse3"
     ,   avx   = "-mavx"
     ,   avx2  = "-mavx2"
+    ,   fma   = "-mfma"
     ,   neon  = "-mfpu=neon"
     }
     return maps[extension]
@@ -268,7 +276,11 @@ end
 
 -- make the link flag
 function nf_link(self, lib)
-    return "-l" .. lib
+    if lib:endswith(".a") or lib:endswith(".so") or lib:endswith(".dylib") or lib:endswith(".lib") then
+        return lib
+    else
+        return "-l" .. lib
+    end
 end
 
 -- make the syslink flag
@@ -323,7 +335,7 @@ end
 
 -- make the c precompiled header flag
 function nf_pcheader(self, pcheaderfile, target)
-    if self:kind() == "cc" or self:kind() == "mm" then
+    if self:kind() == "cc" then
         local pcoutputfile = target:pcoutputfile("c")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -335,7 +347,7 @@ end
 
 -- make the c++ precompiled header flag
 function nf_pcxxheader(self, pcheaderfile, target)
-    if self:kind() == "cxx" or self:kind() == "mxx" then
+    if self:kind() == "cxx" then
         local pcoutputfile = target:pcoutputfile("cxx")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -345,6 +357,30 @@ function nf_pcxxheader(self, pcheaderfile, target)
     end
 end
 
+-- make the objc precompiled header flag
+function nf_pmheader(self, pcheaderfile, target)
+    if self:kind() == "mm" then
+        local pcoutputfile = target:pcoutputfile("m")
+        if self:name() == "clang" then
+            return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
+        else
+            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+        end
+    end
+end
+
+-- make the objc++ precompiled header flag
+function nf_pmxxheader(self, pcheaderfile, target)
+    if self:kind() == "mxx" then
+        local pcoutputfile = target:pcoutputfile("mxx")
+        if self:name() == "clang" then
+            return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
+        else
+            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+        end
+    end
+end
+
 -- add the special flags for the given source file of target
 --
 -- @note only it called when fileconfig is set
@@ -619,6 +655,9 @@ function _compargv_pch(self, pcheaderfile, pcoutputfile, flags)
     if self:kind() == "cxx" then
         table.insert(pchflags, "-x")
         table.insert(pchflags, "c++-header")
+    elseif self:kind() == "mxx" then
+        table.insert(pchflags, "-x")
+        table.insert(pchflags, "objective-c++-header")
     end
 
     -- make the compile arguments list
