@@ -515,7 +515,10 @@ function _finish_requireinfo(requireinfo, package)
     end
     -- we need ensure readonly configs
     for _, name in ipairs(table.keys(requireinfo.configs)) do
-        if package:extraconf("configs", name, "readonly") then
+        local current = requireinfo.configs[name]
+        local default = package:extraconf("configs", name, "default")
+        if package:extraconf("configs", name, "readonly") and current ~= default then
+            wprint("configs.%s is readonly in package(%s), it's always %s", name, package:name(), default)
             -- package:config() will use default value after loading package
             requireinfo.configs[name] = nil
         end
